/**
 * Copyright (C) 1998-2011 enStratusNetworks LLC
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.persist;

public final class Key {
    private String[] fields;
    
    public Key(String ... fields) {
        this.fields = fields;
    }
    
    public boolean equals(Object ob) {
        if( ob == null ) {
            return false;
        }
        if( ob == this ) {
            return true;
        }
        if( !(ob instanceof Key) ) {
            return false;
        }
        Key k = (Key)ob;
        if( k.fields.length != fields.length ) {
            return false;
        }
        for( int i=0; i<fields.length; i++ ) {
            if( !fields[i].equals(k.fields[i]) ) {
                return false;
            }
        }
        return true;
    }
    
    public String[] getFields() {
        return fields;
    }
    
    public boolean matches(SearchTerm ... terms) {
        if( terms == null || terms.length != fields.length ) {
            return false;
        }
        for( int i=0; i<terms.length; i++ ) {
            if( !terms[i].getColumn().equals(fields[i]) ) {
                return false;
            }
        }
        return true;
    }
}
